@@ -17,16 +17,45 @@
 # ------------------------------------------------------------------------
 
 import operator
-from typing import Any, Callable, List, Union
+from abc import abstractmethod
+from typing import Callable, List, Union
 
 import numpy as np
 import PIL
 
-from .. import util as U
+from .. import util
 from . import image_filters_functional as F
 
+try:
+    from typing import Protocol, runtime_checkable
+except ImportError:
+    from typing_extensions import Protocol, runtime_checkable
 
-class Compose(object):
+
+@runtime_checkable
+class Filter(Protocol):
+    """Filter protocol"""
+
+    @abstractmethod
+    def __call__(
+        self, img: Union[PIL.Image.Image, np.ndarray]
+    ) -> Union[PIL.Image.Image, np.ndarray]:
+        raise NotImplementedError
+
+    def __repr__(self) -> str:
+        return self.__class__.__name__ + "()"
+
+
+@runtime_checkable
+class ImageFilter(Filter, Protocol):
+    """Image filter protocol"""
+
+    @abstractmethod
+    def __call__(self, img: PIL.Image.Image) -> Union[PIL.Image.Image, np.ndarray]:
+        raise NotImplementedError
+
+
+class Compose(ImageFilter):
     """Composes several filters together.
 
     Parameters
@@ -35,16 +64,16 @@ class Compose(object):
         List of filters to compose
     """
 
-    def __init__(self, filters: List[Any]) -> None:
+    def __init__(self, filters: List[Filter]) -> None:
         self.filters = filters
 
     def __call__(self, img: PIL.Image.Image) -> Union[PIL.Image.Image, np.ndarray]:
-        for f in self.filters:
-            img = f(img)
+        for filter_ in self.filters:
+            img = filter_(img)
         return img
 
 
-class Lambda(object):
+class Lambda(ImageFilter):
     """Apply a user-defined lambda as a filter.
 
     Inspired from:
@@ -68,11 +97,8 @@ class Lambda(object):
     def __call__(self, img: PIL.Image.Image) -> Union[PIL.Image.Image, np.ndarray]:
         return self.lambd(img)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class ToPILImage(object):
+class ToPILImage(Filter):
     """Convert a ndarray to a PIL Image, while preserving the value range.
 
     Parameters
@@ -87,13 +113,10 @@ class ToPILImage(object):
     """
 
     def __call__(self, np_img: np.ndarray) -> PIL.Image.Image:
-        return U.np_to_pil(np_img)
-
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
+        return util.np_to_pil(np_img)
 
 
-class ApplyMaskImage(object):
+class ApplyMaskImage(Filter):
     """Mask image with the provided binary mask.
 
     Parameters
@@ -113,13 +136,10 @@ class ApplyMaskImage(object):
         self.img = img
 
     def __call__(self, mask: np.ndarray) -> PIL.Image.Image:
-        return U.apply_mask_image(self.img, mask)
-
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
+        return util.apply_mask_image(self.img, mask)
 
 
-class Invert(object):
+class Invert(ImageFilter):
     """Invert an image, i.e. take the complement of the correspondent array.
 
     Parameters
@@ -136,11 +156,8 @@ class Invert(object):
     def __call__(self, img: PIL.Image.Image) -> PIL.Image.Image:
         return F.invert(img)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class RgbToGrayscale(object):
+class RgbToGrayscale(ImageFilter):
     """Convert an RGB image to a grayscale image.
 
     Parameters
@@ -157,11 +174,8 @@ class RgbToGrayscale(object):
     def __call__(self, img: PIL.Image.Image) -> PIL.Image.Image:
         return PIL.ImageOps.grayscale(img)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class RgbToHed(object):
+class RgbToHed(ImageFilter):
     """Convert RGB channels to HED channels.
 
     image color space (RGB) is converted to Hematoxylin-Eosin-Diaminobenzidine space.
@@ -181,11 +195,42 @@ class RgbToHed(object):
         hed = F.rgb_to_hed(img)
         return hed
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
+
+class RgbToLab(ImageFilter):
+    """Convert from the sRGB color space to the CIE Lab colorspace.
+
+    sRGB color space reference: IEC 61966-2-1:1999
+
+    Parameters
+    ----------
+    img : PIL.Image.Image
+        Input image
+    illuminant : {“A”, “D50”, “D55”, “D65”, “D75”, “E”}, optional
+        The name of the illuminant (the function is NOT case sensitive).
+    observer : {“2”, “10”}, optional
+        The aperture angle of the observer.
+
+    Returns
+    -------
+    PIL.Image.Image
+        Image in LAB space
+
+    Raises
+    ------
+    Exception
+        If the image mode is not RGB
+    """
+
+    def __init__(self, illuminant: str = "D65", observer: int = "2") -> None:
+        self.illuminant = illuminant
+        self.observer = observer
+
+    def __call__(self, img: PIL.Image.Image) -> PIL.Image.Image:
+        lab = F.rgb_to_lab(img)
+        return lab
 
 
-class HematoxylinChannel(object):
+class HematoxylinChannel(ImageFilter):
     """Obtain Hematoxylin channel from RGB image.
 
     Input image is first converted into HED space and the hematoxylin channel is
@@ -206,11 +251,8 @@ class HematoxylinChannel(object):
         hematoxylin = F.hematoxylin_channel(img)
         return hematoxylin
 
-    def __repr__(self):
-        return self.__class__.__name__ + "()"
 
-
-class EosinChannel(object):
+class EosinChannel(ImageFilter):
     """Obtain Eosin channel from RGB image.
 
     Input image is first converted into HED space and the Eosin channel is
@@ -231,11 +273,8 @@ class EosinChannel(object):
         eosin = F.eosin_channel(img)
         return eosin
 
-    def __repr__(self):
-        return self.__class__.__name__ + "()"
 
-
-class RgbToHsv(object):
+class RgbToHsv(ImageFilter):
     """Convert RGB channels to HSV channels.
 
     image color space (RGB) is converted to Hue - Saturation - Value (HSV) space.
@@ -255,11 +294,8 @@ class RgbToHsv(object):
         hsv = F.rgb_to_hsv(img)
         return hsv
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class StretchContrast(object):
+class StretchContrast(ImageFilter):
     """Increase image contrast.
 
     Th contrast in image is increased based on intensities in a specified range
@@ -287,11 +323,8 @@ class StretchContrast(object):
         stretch_contrast = F.stretch_contrast(img, self.low, self.high)
         return stretch_contrast
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class HistogramEqualization(object):
+class HistogramEqualization(ImageFilter):
     """Increase image contrast using histogram equalization.
 
     The input image (gray or RGB) is filterd using histogram equalization to increase
@@ -317,11 +350,8 @@ class HistogramEqualization(object):
         hist_equ = F.histogram_equalization(img, self.n_bins)
         return hist_equ
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class AdaptiveEqualization(object):
+class AdaptiveEqualization(ImageFilter):
     """Increase image contrast using adaptive equalization.
 
     Contrast in local region of input image (gray or RGB) is increased using
@@ -350,14 +380,8 @@ class AdaptiveEqualization(object):
         adaptive_equ = F.adaptive_equalization(img, self.n_bins, self.clip_limit)
         return adaptive_equ
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-# ------- LocalEqualization input must be 2D (grayscale)
-
-
-class LocalEqualization(object):
+class LocalEqualization(ImageFilter):
     """Filter gray image using local equalization.
 
     Local equalization method uses local histograms based on a disk structuring element.
@@ -365,7 +389,7 @@ class LocalEqualization(object):
     Parameters
     ---------
     img: PIL.Image.Image
-        Input image. Notice that it must be 2D
+        Grayscale input image
     disk_size: int, optional
         Radius of the disk structuring element used for the local histograms.
         Default is 50
@@ -373,7 +397,7 @@ class LocalEqualization(object):
     Returns
     -------
     PIL.Image.Image
-        2D image with contrast enhanced using local equalization.
+        Grayscale image with contrast enhanced using local equalization.
     """
 
     def __init__(self, disk_size: int = 50) -> None:
@@ -383,11 +407,8 @@ class LocalEqualization(object):
         local_equ = F.local_equalization(img, self.disk_size)
         return local_equ
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class KmeansSegmentation(object):
+class KmeansSegmentation(ImageFilter):
     """Segment an RGB image with K-means segmentation
 
     By using K-means segmentation (color/space proximity) each segment is colored based
@@ -419,11 +440,8 @@ class KmeansSegmentation(object):
         )
         return kmeans_segmentation
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class RagThreshold(object):
+class RagThreshold(ImageFilter):
     """Combine similar K-means segmented regions based on threshold value.
 
     Segment an image with K-means, build region adjacency graph based on
@@ -458,11 +476,8 @@ class RagThreshold(object):
     def __call__(self, img: PIL.Image.Image) -> PIL.Image.Image:
         return F.rag_threshold(img, self.n_segments, self.compactness, self.threshold)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class HysteresisThreshold(object):
+class HysteresisThreshold(ImageFilter):
     """Apply two-level (hysteresis) threshold to an image.
 
     Parameters
@@ -487,14 +502,11 @@ class HysteresisThreshold(object):
     def __call__(self, img: PIL.Image.Image) -> PIL.Image.Image:
         return F.hysteresis_threshold(img, self.low, self.high)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
-
 
 # ----------- Branching functions (grayscale/invert input)-------------------
 
 # invert --> grayscale ..> hysteresis
-class HysteresisThresholdMask(object):
+class HysteresisThresholdMask(ImageFilter):
     """Mask an image using hysteresis threshold
 
     Compute the Hysteresis threshold on the complement of a grayscale image,
@@ -525,11 +537,8 @@ class HysteresisThresholdMask(object):
         )
         return hysteresis_threshold_mask
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class OtsuThreshold(object):
+class OtsuThreshold(ImageFilter):
     """Mask image based on pixel above Otsu threshold.
 
     Compute Otsu threshold on image as a NumPy array and return boolean mask
@@ -551,11 +560,8 @@ class OtsuThreshold(object):
     def __call__(self, img: PIL.Image.Image) -> np.ndarray:
         return F.otsu_threshold(img)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class LocalOtsuThreshold(object):
+class LocalOtsuThreshold(ImageFilter):
     """Mask image based on local Otsu threshold.
 
     Compute local Otsu threshold for each pixel and return boolean mask
@@ -583,11 +589,8 @@ class LocalOtsuThreshold(object):
     def __call__(self, img: PIL.Image.Image) -> np.ndarray:
         return F.local_otsu_threshold(img, self.disk_size)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class FilterEntropy(object):
+class FilterEntropy(ImageFilter):
     """Filter image based on entropy (complexity).
 
     The area of the image included in the local neighborhood is defined by a square
@@ -617,11 +620,8 @@ class FilterEntropy(object):
     def __call__(self, img: PIL.Image.Image) -> np.ndarray:
         return F.filter_entropy(img, self.neighborhood, self.threshold)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class CannyEdges(object):
+class CannyEdges(ImageFilter):
     """Filter image based on Canny edge algorithm.
 
     Note that input image must be 2D
@@ -656,11 +656,8 @@ class CannyEdges(object):
     def __call__(self, img: PIL.Image.Image) -> np.ndarray:
         return F.canny_edges(img, self.sigma, self.low_threshold, self.high_threshold)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class Grays(object):
+class Grays(ImageFilter):
     """Filter out gray pixels in RGB image.
 
     Gray pixels are those pixels where the red, green, and blue channel values
@@ -686,11 +683,8 @@ class Grays(object):
     def __call__(self, img: PIL.Image.Image) -> PIL.Image.Image:
         return F.grays(img, self.tolerance)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class GreenChannelFilter(object):
+class GreenChannelFilter(ImageFilter):
     """Mask pixels in an RGB image with G-channel greater than a specified threshold.
 
     Create a mask to filter out pixels with a green channel value greater than
@@ -732,11 +726,8 @@ class GreenChannelFilter(object):
             img, self.green_thresh, self.avoid_overmask, self.overmask_thresh
         )
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class RedFilter(object):
+class RedFilter(ImageFilter):
     """Mask reddish colors in an RGB image.
 
     Create a mask to filter out reddish colors, where the mask is based on a pixel
@@ -768,11 +759,8 @@ class RedFilter(object):
     def __call__(self, img: PIL.Image.Image) -> np.ndarray:
         return F.red_filter(img, self.red_thresh, self.green_thresh, self.blue_thresh)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class RedPenFilter(object):
+class RedPenFilter(ImageFilter):
     """Filter out red pen marks on diagnostic slides.
 
     The resulting mask is a composition of red filters with different thresholds
@@ -791,11 +779,8 @@ class RedPenFilter(object):
     def __call__(self, img: PIL.Image.Image):
         return F.red_pen_filter(img)
 
-    def __repr__(self):
-        return self.__class__.__name__ + "()"
 
-
-class GreenFilter(object):
+class GreenFilter(ImageFilter):
     """Filter out greenish colors in an RGB image.
     The mask is based on a pixel being above a red channel threshold value, below a
     green channel threshold value, and below a blue channel threshold value.
@@ -828,11 +813,8 @@ class GreenFilter(object):
     def __call__(self, img):
         return F.green_filter(img, self.red_thresh, self.green_thresh, self.blue_thresh)
 
-    def __repr__(self):
-        return self.__class__.__name__ + "()"
 
-
-class GreenPenFilter(object):
+class GreenPenFilter(ImageFilter):
     """Filter out green pen marks from a diagnostic slide.
 
     The resulting mask is a composition of green filters with different thresholds
@@ -852,11 +834,8 @@ class GreenPenFilter(object):
     def __call__(self, img: PIL.Image.Image) -> PIL.Image.Image:
         return F.green_pen_filter(img)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class BlueFilter(object):
+class BlueFilter(ImageFilter):
     """Filter out blueish colors in an RGB image.
 
     Create a mask to filter out blueish colors, where the mask is based on a pixel
@@ -888,11 +867,8 @@ class BlueFilter(object):
     def __call__(self, img: PIL.Image.Image) -> np.ndarray:
         return F.blue_filter(img, self.red_thresh, self.green_thresh, self.blue_thresh)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class BluePenFilter(object):
+class BluePenFilter(ImageFilter):
     """Filter out blue pen marks from a diagnostic slide.
 
     The resulting mask is a composition of green filters with different thresholds
@@ -912,35 +888,8 @@ class BluePenFilter(object):
     def __call__(self, img: PIL.Image.Image) -> PIL.Image.Image:
         return F.blue_pen_filter(img)
 
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
 
-
-class PenMarks(object):
-    """Filter out pen marks from a diagnostic slide.
-
-    Pen marks are removed by applying Otsu threshold on the H channel of the image
-    converted to the HSV space.
-
-    Parameters
-    ---------
-    img : PIL.Image.Image
-        Input RGB image
-
-    Returns
-    -------
-    np.ndarray
-        Boolean NumPy array representing the mask with the pen marks filtered out.
-    """
-
-    def __call__(self, img: PIL.Image.Image) -> PIL.Image.Image:
-        return F.pen_marks(img)
-
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
-
-
-class YenThreshold(object):
+class YenThreshold(ImageFilter):
     """Mask image based on pixel above Yen threshold.
 
     Compute Yen threshold on image and return boolean mask based on pixels below this
@@ -965,6 +914,3 @@ class YenThreshold(object):
 
     def __call__(self, img: PIL.Image.Image) -> np.ndarray:
         return F.yen_threshold(img, self.relate)
-
-    def __repr__(self) -> str:
-        return self.__class__.__name__ + "()"
