@@ -24,7 +24,8 @@ Slide is the main API class for manipulating slide objects.
 import math
 import os
 import pathlib
-from typing import List, Tuple, Union
+from functools import lru_cache
+from typing import Iterator, List, Tuple, Union
 
 import ntpath
 import numpy as np
@@ -37,7 +38,6 @@ from .tile import Tile
 from .types import CoordinatePair, Region
 from .util import (
     lazyproperty,
-    lru_cache,
     polygon_to_mask_array,
     region_coordinates,
     regions_from_binary_mask,
@@ -47,7 +47,7 @@ from .util import (
 IMG_EXT = "png"
 
 
-class Slide(object):
+class Slide:
     """Provide Slide objects and expose property and methods.
 
     Arguments
@@ -120,6 +120,7 @@ class Slide(object):
             Image containing the selected tile.
         """
 
+        level = level if level >= 0 else self._remap_level(level)
         if not self._has_valid_coords(coords):
             # OpenSlide doesn't complain if the coordinates for extraction are wrong,
             # but it returns an odd image.
@@ -155,6 +156,7 @@ class Slide(object):
         -------
         dimensions : tuple (width, height)
         """
+        level = level if level >= 0 else self._remap_level(level)
         try:
             return self._wsi.level_dimensions[level]
         except IndexError:
@@ -174,6 +176,55 @@ class Slide(object):
         """
         return list(range(len(self._wsi.level_dimensions)))
 
+    def locate_biggest_tissue_box(
+        self,
+        scale_factor: int = 32,
+        tissue_mask: bool = False,
+        alpha: int = 128,
+        outline: str = "red",
+    ) -> PIL.Image.Image:
+        """Draw biggest tissue box reference on a rescaled version of the slide
+
+        Parameters
+        ----------
+        scale_factor : int
+            Scaling factor for the returned image. Default is 32.
+        tissue_mask : bool, optional
+            Whether to draw the biggest tissue box on the binary tissue mask instead of
+            the rescaled version of the slide. Default is False.
+        alpha : int
+            The alpha level to be applied to the rescaled slide, default to 128.
+        outline : str
+            The outline color for the annotation, default to 'red'.
+
+        Returns
+        -------
+        PIL.Image.Image
+            PIL Image of the rescaled slide with the biggest tissue bounding box
+            outlined.
+        """
+        if not os.path.exists(self.scaled_image_path(scale_factor)):
+            self.save_scaled_image(scale_factor)
+        img = PIL.Image.open(self.scaled_image_path(scale_factor))
+
+        filters = FiltersComposition(Slide).tissue_mask_filters
+
+        img_tissue_mask = filters(img)
+        regions = regions_from_binary_mask(img_tissue_mask)
+        biggest_region = self._biggest_regions(regions, n=1)[0]
+
+        biggest_region_coordinates = region_coordinates(biggest_region)
+
+        if tissue_mask:
+            img = PIL.Image.fromarray(img_tissue_mask).convert("RGB")
+        else:
+            img.putalpha(alpha)
+
+        draw = PIL.ImageDraw.Draw(img)
+        draw.rectangle(tuple(biggest_region_coordinates), outline=outline)
+
+        return img
+
     @lazyproperty
     def name(self) -> str:
         """Retrieve the slide name without extension.
@@ -195,6 +246,17 @@ class Slide(object):
         """
         return self._processed_path
 
+    @lazyproperty
+    def properties(self) -> dict:
+        """Whole Slide Image properties.
+
+        Returns
+        -------
+        dict
+            WSI complete properties.
+        """
+        return dict(self._wsi.properties)
+
     def resampled_array(self, scale_factor: int = 32) -> np.array:
         """Retrieve the resampled array from the original slide
 
@@ -243,7 +305,7 @@ class Slide(object):
         -------
         img_path : str
         """
-        img_path = self._breadcumb(self._processed_path, scale_factor)
+        img_path = self._breadcrumb(self._processed_path, scale_factor)
         return img_path
 
     def show(self) -> None:
@@ -254,8 +316,8 @@ class Slide(object):
         try:
             thumbnail = PIL.Image.open(self.thumbnail_path)
             thumbnail.show()  # pragma: no cover
-        except FileNotFoundError as e:
-            raise FileNotFoundError(f"Cannot display the slide thumbnail:{e}")
+        except FileNotFoundError as error:
+            raise FileNotFoundError(f"Cannot display the slide thumbnail:{error}")
 
     @lazyproperty
     def thumbnail_path(self) -> str:
@@ -272,7 +334,8 @@ class Slide(object):
 
     # ------- implementation helpers -------
 
-    def _biggest_regions(self, regions: List[Region], n: int = 1) -> List[Region]:
+    @staticmethod
+    def _biggest_regions(regions: List[Region], n: int = 1) -> List[Region]:
         """Return the biggest ``n`` regions.
 
         Parameters
@@ -299,7 +362,7 @@ class Slide(object):
         sorted_regions = sorted(regions, key=lambda r: r.area, reverse=True)
         return sorted_regions[:n]
 
-    def _breadcumb(self, directory_path: str, scale_factor: int = 32) -> str:
+    def _breadcrumb(self, directory_path: str, scale_factor: int = 32) -> str:
         """Returns a complete path according to the give directory path
 
         Parameters
@@ -345,6 +408,31 @@ class Slide(object):
             and 0 <= coords.y_br < self.dimensions[1]
         )
 
+    def _remap_level(self, level: int) -> int:
+        """Remap negative index for the given level onto a positive one.
+
+        Parameters
+        ----------
+        level : int
+            the level index to remap
+
+        Raises
+        ------
+        LevelError
+            when the abs(level) is greater than the number of the levels.
+
+        Returns
+        -------
+        level : int
+           positive level index
+        """
+        if len(self.levels) - abs(level) < 0:
+            raise LevelError(
+                f"Level {level} not available. Number of available levels: "
+                f"{len(self._wsi.level_dimensions)}"
+            )
+        return len(self.levels) - abs(level)
+
     def _resample(self, scale_factor: int = 32) -> Tuple[PIL.Image.Image, np.array]:
         """Converts a slide to a scaled-down PIL image.
 
@@ -360,11 +448,10 @@ class Slide(object):
 
         Returns
         -------
-        img, arr_img, large_w, large_h, new_w, new_h: tuple
+        img, arr_img
         """
-        # TODO: use logger instead of print(f"Opening Slide {slide_filepath}")
 
-        large_w, large_h, new_w, new_h = self._resampled_dimensions(scale_factor)
+        _, _, new_w, new_h = self._resampled_dimensions(scale_factor)
         level = self._wsi.get_best_level_for_downsample(scale_factor)
         whole_slide_image = self._wsi.read_region(
             (0, 0), level, self._wsi.level_dimensions[level]
@@ -441,16 +528,16 @@ class Slide(object):
         """
         try:
             slide = openslide.open_slide(self._path)
-        except openslide.OpenSlideError:
-            raise openslide.OpenSlideError(
+        except PIL.UnidentifiedImageError:
+            raise PIL.UnidentifiedImageError(
                 "Your wsi has something broken inside, a doctor is needed"
-            )  # pragma: no cover
+            )
         except FileNotFoundError:
             raise FileNotFoundError("The wsi path resource doesn't exist")
         return slide
 
 
-class SlideSet(object):
+class SlideSet:
     """Slideset object. It is considered a collection of Slides."""
 
     def __init__(
@@ -460,6 +547,35 @@ class SlideSet(object):
         self._processed_path = processed_path
         self._valid_extensions = valid_extensions
 
+    def __iter__(self) -> Iterator[Slide]:
+        """Slides of the slideset
+
+        Returns
+        -------
+        generator of `Slide` objects.
+        """
+        return iter(
+            [
+                Slide(os.path.join(self._slides_path, _path), self._processed_path)
+                for _path in os.listdir(self._slides_path)
+                if os.path.splitext(_path)[1] in self._valid_extensions
+            ]
+        )
+
+    def __getitem__(self, slide_id: int) -> Slide:
+        """Slide object given the correspondent id"""
+        return list(self.__iter__())[slide_id]
+
+    def __len__(self) -> int:
+        """Total number of the slides of this Slideset
+
+        Returns
+        -------
+        int
+            number of the Slides.
+        """
+        return len(list(self.__iter__()))
+
     # ---public interface methods and properties---
 
     def save_scaled_slides(self, scale_factor: int = 32, n: int = 0) -> None:
@@ -475,7 +591,7 @@ class SlideSet(object):
         """
         os.makedirs(self._processed_path, exist_ok=True)
         n = self.total_slides if (n > self.total_slides or n == 0) else n
-        for slide in self.slides[:n]:
+        for slide in list(self.__iter__())[:n]:
             slide.save_scaled_image(scale_factor)
 
     def save_thumbnails(self, n: int = 0) -> None:
@@ -486,29 +602,12 @@ class SlideSet(object):
         n: int. optional
             First n slides in dataset folder. Default is 0, meaning that the thumbnails
             of all the slides will be saved.
-        scale_factor : int, optional
-            Image scaling factor. Default is 32.
         """
-        # TODO: add logger n>total_slide and log thumbnails names
         os.makedirs(self._processed_path, exist_ok=True)
         n = self.total_slides if (n > self.total_slides or n == 0) else n
-        for slide in self.slides[:n]:
+        for slide in list(self.__iter__())[:n]:
             slide.save_thumbnail()
 
-    @lazyproperty
-    def slides(self) -> List[Slide]:
-        """Retrieve all the slides of the slideset
-
-        Returns
-        ----------
-        slides: list, list of `Slide` objects
-        """
-        return [
-            Slide(os.path.join(self._slides_path, _path), self._processed_path)
-            for _path in os.listdir(self._slides_path)
-            if os.path.splitext(_path)[1] in self._valid_extensions
-        ]
-
     @lazyproperty
     def slides_stats(self) -> dict:
         """Retrieve statistic/graphs of slides files contained in the dataset.
@@ -539,7 +638,7 @@ class SlideSet(object):
         n: int
             Number of slides.
         """
-        return len(self.slides)
+        return self.__len__()
 
     # ---private interface methods and properties---
 
@@ -594,9 +693,9 @@ class SlideSet(object):
                 "height": slide.dimensions[1],
                 "size": slide.dimensions[0] * slide.dimensions[1],
             }
-            for slide in self.slides
+            for slide in list(self.__iter__())
         ]
 
     @lazyproperty
     def _slides_dimensions_list(self):
-        return [slide.dimensions for slide in self.slides]
+        return [slide.dimensions for slide in list(self.__iter__())]
