@@ -1,17 +1,36 @@
+# encoding: utf-8
+
+# ------------------------------------------------------------------------
+# Copyright 2020 All Histolab Contributors
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+# ------------------------------------------------------------------------
+
 import csv
 import os
 from abc import abstractmethod
+from functools import lru_cache
 from typing import List, Tuple
 
 import numpy as np
+import PIL
 
-from .exceptions import LevelError
+from .exceptions import LevelError, TileSizeError
 from .scorer import Scorer
 from .slide import Slide
 from .tile import Tile
 from .types import CoordinatePair
 from .util import (
-    lru_cache,
     region_coordinates,
     regions_from_binary_mask,
     scale_coordinates,
@@ -25,6 +44,7 @@ except ImportError:
 
 @runtime_checkable
 class Tiler(Protocol):
+    """General tiler object"""
 
     level: int
     tile_size: int
@@ -45,15 +65,75 @@ class Tiler(Protocol):
         np.ndarray
             Extraction mask at thumbnail level
         """
-
         return slide.biggest_tissue_box_mask
 
     @abstractmethod
     def extract(self, slide: Slide):
         raise NotImplementedError
 
+    def locate_tiles(
+        self,
+        slide: Slide,
+        scale_factor: int = 32,
+        alpha: int = 128,
+        outline: str = "red",
+    ) -> PIL.Image.Image:
+        """Draw tile box references on a rescaled version of the slide
+
+        Parameters
+        ----------
+        slide : Slide
+            Slide reference where placing the tiles
+        scale_factor: int
+            Scaling factor for the returned image. Default is 32.
+        alpha: int
+            The alpha level to be applied to the rescaled slide, default to 128.
+        outline: str
+            The outline color for the tile annotations, default to 'red'.
+
+        Returns
+        -------
+        PIL.Image.Image
+            PIL Image of the rescaled slide with the extracted tiles outlined
+        """
+        if not os.path.exists(slide.scaled_image_path(scale_factor)):
+            slide.save_scaled_image(scale_factor)
+        img = PIL.Image.open(slide.scaled_image_path(scale_factor))
+        img.putalpha(alpha)
+        draw = PIL.ImageDraw.Draw(img)
+
+        tiles = (
+            self._tiles_generator(slide)[0]
+            if isinstance(self, ScoreTiler)
+            else self._tiles_generator(slide)
+        )
+        tiles_coords = (tile[1] for tile in tiles)
+        for coords in tiles_coords:
+            rescaled = scale_coordinates(coords, slide.dimensions, img.size)
+            draw.rectangle(tuple(rescaled), outline=outline)
+        return img
+
     # ------- implementation helpers -------
 
+    def _has_valid_tile_size(self, slide: Slide) -> bool:
+        """Return True if the tile size is smaller or equal than the ``slide`` size.
+
+        Parameters
+        ----------
+        slide : Slide
+            The slide to check the tile size against.
+
+        Returns
+        -------
+        bool
+            True if the tile size is smaller or equal than the ``slide`` size at
+            extraction level, False otherwise
+        """
+        return (
+            self.tile_size[0] <= slide.level_dimensions(self.level)[0]
+            and self.tile_size[1] <= slide.level_dimensions(self.level)[1]
+        )
+
     def _tile_filename(
         self, tile_wsi_coords: CoordinatePair, tiles_counter: int
     ) -> str:
@@ -73,7 +153,6 @@ class Tiler(Protocol):
             `{prefix}tile_{tiles_counter}_level{level}_{x_ul_wsi}-{y_ul_wsi}-{x_br_wsi}"
             "-{y_br_wsi}{suffix}`
         """
-
         x_ul_wsi, y_ul_wsi, x_br_wsi, y_br_wsi = tile_wsi_coords
         tile_filename = (
             f"{self.prefix}tile_{tiles_counter}_level{self.level}_{x_ul_wsi}-{y_ul_wsi}"
@@ -82,6 +161,47 @@ class Tiler(Protocol):
 
         return tile_filename
 
+    def _tiles_generator(self, slide: Slide) -> None:
+        raise NotImplementedError
+
+    def _validate_level(self, slide: Slide) -> None:
+        """Validate the Tiler's level according to the Slide.
+
+        Parameters
+        ----------
+        slide : Slide
+            Slide from which to extract the tiles
+
+        Raises
+        ------
+        LevelError
+            If the level is not available for the slide
+        """
+        if len(slide.levels) - abs(self.level) < 0:
+            raise LevelError(
+                f"Level {self.level} not available. Number of available levels: "
+                f"{len(slide.levels)}"
+            )
+
+    def _validate_tile_size(self, slide: Slide) -> None:
+        """Validate the tile size according to the Slide.
+
+        Parameters
+        ----------
+        slide : Slide
+            Slide from which to extract the tiles
+
+        Raises
+        ------
+        TileSizeError
+            If the tile size is larger than the slide size
+        """
+        if not self._has_valid_tile_size(slide):
+            raise TileSizeError(
+                f"Tile size {self.tile_size} is larger than slide size "
+                f"{slide.level_dimensions(self.level)} at level {self.level}"
+            )
+
 
 class GridTiler(Tiler):
     """Extractor of tiles arranged in a grid, at the given level, with the given size.
@@ -94,6 +214,10 @@ class GridTiler(Tiler):
         Level from which extract the tiles. Default is 0.
     check_tissue : bool, optional
         Whether to check if the tile has enough tissue to be saved. Default is True.
+    tissue_percent : float, optional
+        Number between 0.0 and 100.0 representing the minimum required percentage of
+        tissue over the total area of the image, default is 80.0. This is considered
+        only if ``check_tissue`` equals to True.
     pixel_overlap : int, optional
        Number of overlapping pixels (for both height and width) between two adjacent
        tiles. If negative, two adjacent tiles will be strided by the absolute value of
@@ -109,6 +233,7 @@ class GridTiler(Tiler):
         tile_size: Tuple[int, int],
         level: int = 0,
         check_tissue: bool = True,
+        tissue_percent: float = 80.0,
         pixel_overlap: int = 0,
         prefix: str = "",
         suffix: str = ".png",
@@ -116,11 +241,12 @@ class GridTiler(Tiler):
         self.tile_size = tile_size
         self.level = level
         self.check_tissue = check_tissue
+        self.tissue_percent = tissue_percent
         self.pixel_overlap = pixel_overlap
         self.prefix = prefix
         self.suffix = suffix
 
-    def extract(self, slide: Slide):
+    def extract(self, slide: Slide) -> None:
         """Extract tiles arranged in a grid and save them to disk, following this
         filename pattern:
         `{prefix}tile_{tiles_counter}_level{level}_{x_ul_wsi}-{y_ul_wsi}-{x_br_wsi}-{y_br_wsi}{suffix}`
@@ -129,35 +255,27 @@ class GridTiler(Tiler):
         ----------
         slide : Slide
             Slide from which to extract the tiles
-        """
-        if self.level not in slide.levels:
-            raise LevelError(
-                f"Level {self.level} not available. Number of available levels: "
-                f"{len(slide.levels)}"
-            )
 
-        grid_tiles = self._grid_tiles_generator(slide)
+        Raises
+        ------
+        TileSizeError
+            If the tile size is larger than the slide size
+        LevelError
+            If the level is not available for the slide
+        """
+        self._validate_level(slide)
+        self._validate_tile_size(slide)
+
+        grid_tiles = self._tiles_generator(slide)
 
         tiles_counter = 0
-
         for tiles_counter, (tile, tile_wsi_coords) in enumerate(grid_tiles):
             tile_filename = self._tile_filename(tile_wsi_coords, tiles_counter)
             full_tile_path = os.path.join(slide.processed_path, "tiles", tile_filename)
             tile.save(full_tile_path)
             print(f"\t Tile {tiles_counter} saved: {tile_filename}")
-
         print(f"{tiles_counter} Grid Tiles have been saved.")
 
-    @property
-    def level(self) -> int:
-        return self._valid_level
-
-    @level.setter
-    def level(self, level_: int):
-        if level_ < 0:
-            raise LevelError(f"Level cannot be negative ({level_})")
-        self._valid_level = level_
-
     @property
     def tile_size(self) -> Tuple[int, int]:
         return self._valid_tile_size
@@ -194,8 +312,8 @@ class GridTiler(Tiler):
 
         for i in range(n_tiles_row):
             for j in range(n_tiles_column):
-                x_ul_lvl = bbox_coordinates.x_ul + tile_w_lvl * j - self.pixel_overlap
-                y_ul_lvl = bbox_coordinates.y_ul + tile_h_lvl * i - self.pixel_overlap
+                x_ul_lvl = bbox_coordinates.x_ul + tile_w_lvl * i - self.pixel_overlap
+                y_ul_lvl = bbox_coordinates.y_ul + tile_h_lvl * j - self.pixel_overlap
 
                 x_ul_lvl = np.clip(x_ul_lvl, bbox_coordinates.x_ul, None)
                 y_ul_lvl = np.clip(y_ul_lvl, bbox_coordinates.y_ul, None)
@@ -241,7 +359,7 @@ class GridTiler(Tiler):
                 bbox_coordinates, slide
             )
 
-    def _grid_tiles_generator(self, slide: Slide) -> Tuple[Tile, CoordinatePair]:
+    def _tiles_generator(self, slide: Slide) -> Tuple[Tile, CoordinatePair]:
         """Generator of tiles arranged in a grid.
 
         Parameters
@@ -256,7 +374,6 @@ class GridTiler(Tiler):
         CoordinatePair
             Coordinates of the slide at level 0 from which the tile has been extracted
         """
-
         grid_coordinates_generator = self._grid_coordinates_generator(slide)
         for coords in grid_coordinates_generator:
             try:
@@ -264,7 +381,7 @@ class GridTiler(Tiler):
             except ValueError:
                 continue
 
-            if not self.check_tissue or tile.has_enough_tissue():
+            if not self.check_tissue or tile.has_enough_tissue(self.tissue_percent):
                 yield tile, coords
 
     def _n_tiles_column(self, bbox_coordinates: CoordinatePair) -> int:
@@ -318,6 +435,10 @@ class RandomTiler(Tiler):
         is 7.
     check_tissue : bool, optional
         Whether to check if the tile has enough tissue to be saved. Default is True.
+    tissue_percent : float, optional
+        Number between 0.0 and 100.0 representing the minimum required percentage of
+        tissue over the total area of the image, default is 80.0. This is considered
+        only if ``check_tissue`` equals to True.
     prefix : str, optional
         Prefix to be added to the tile filename. Default is an empty string.
     suffix : str, optional
@@ -334,9 +455,10 @@ class RandomTiler(Tiler):
         level: int = 0,
         seed: int = 7,
         check_tissue: bool = True,
+        tissue_percent: float = 80.0,
         prefix: str = "",
         suffix: str = ".png",
-        max_iter: int = 1e4,
+        max_iter: int = int(1e4),
     ):
 
         super().__init__()
@@ -347,10 +469,11 @@ class RandomTiler(Tiler):
         self.level = level
         self.seed = seed
         self.check_tissue = check_tissue
+        self.tissue_percent = tissue_percent
         self.prefix = prefix
         self.suffix = suffix
 
-    def extract(self, slide: Slide):
+    def extract(self, slide: Slide) -> None:
         """Extract random tiles and save them to disk, following this filename pattern:
         `{prefix}tile_{tiles_counter}_level{level}_{x_ul_wsi}-{y_ul_wsi}-{x_br_wsi}-{y_br_wsi}{suffix}`
 
@@ -358,35 +481,33 @@ class RandomTiler(Tiler):
         ----------
         slide : Slide
             Slide from which to extract the tiles
+
+        Raises
+        ------
+        TileSizeError
+            If the tile size is larger than the slide size
+        LevelError
+            If the level is not available for the slide
         """
+        self._validate_level(slide)
+        self._validate_tile_size(slide)
 
-        np.random.seed(self.seed)
-
-        random_tiles = self._random_tiles_generator(slide)
+        random_tiles = self._tiles_generator(slide)
 
         tiles_counter = 0
         for tiles_counter, (tile, tile_wsi_coords) in enumerate(random_tiles):
             tile_filename = self._tile_filename(tile_wsi_coords, tiles_counter)
-            tile.save(os.path.join(slide.processed_path, "tiles", tile_filename))
+            full_tile_path = os.path.join(slide.processed_path, "tiles", tile_filename)
+            tile.save(full_tile_path)
             print(f"\t Tile {tiles_counter} saved: {tile_filename}")
         print(f"{tiles_counter+1} Random Tiles have been saved.")
 
-    @property
-    def level(self) -> int:
-        return self._valid_level
-
-    @level.setter
-    def level(self, level_: int):
-        if level_ < 0:
-            raise LevelError(f"Level cannot be negative ({level_})")
-        self._valid_level = level_
-
     @property
     def max_iter(self) -> int:
         return self._valid_max_iter
 
     @max_iter.setter
-    def max_iter(self, max_iter_: int = 1e4):
+    def max_iter(self, max_iter_: int = int(1e4)):
         if max_iter_ < self.n_tiles:
             raise ValueError(
                 f"The maximum number of iterations ({max_iter_}) must be grater than or"
@@ -444,7 +565,7 @@ class RandomTiler(Tiler):
 
         return tile_wsi_coords
 
-    def _random_tiles_generator(self, slide: Slide) -> Tuple[Tile, CoordinatePair]:
+    def _tiles_generator(self, slide: Slide) -> Tuple[Tile, CoordinatePair]:
         """Generate Random Tiles within a slide box.
 
         Stops if:
@@ -463,11 +584,10 @@ class RandomTiler(Tiler):
         coords : CoordinatePair
             The level-0 coordinates of the extracted tile
         """
-
+        np.random.seed(self.seed)
         iteration = valid_tile_counter = 0
 
         while True:
-
             tile_wsi_coords = self._random_tile_coordinates(slide)
             try:
                 tile = slide.extract_tile(tile_wsi_coords, self.level)
@@ -475,7 +595,7 @@ class RandomTiler(Tiler):
                 iteration -= 1
                 continue
 
-            if not self.check_tissue or tile.has_enough_tissue():
+            if not self.check_tissue or tile.has_enough_tissue(self.tissue_percent):
                 yield tile, tile_wsi_coords
                 valid_tile_counter += 1
             iteration += 1
@@ -506,6 +626,10 @@ class ScoreTiler(GridTiler):
         Level from which extract the tiles. Default is 0.
     check_tissue : bool, optional
         Whether to check if the tile has enough tissue to be saved. Default is True.
+    tissue_percent : float, optional
+        Number between 0.0 and 100.0 representing the minimum required percentage of
+        tissue over the total area of the image, default is 80.0. This is considered
+        only if ``check_tissue`` equals to True.
     pixel_overlap : int, optional
        Number of overlapping pixels (for both height and width) between two adjacent
        tiles. If negative, two adjacent tiles will be strided by the absolute value of
@@ -523,6 +647,7 @@ class ScoreTiler(GridTiler):
         n_tiles: int = 0,
         level: int = 0,
         check_tissue: bool = True,
+        tissue_percent: float = 80.0,
         pixel_overlap: int = 0,
         prefix: str = "",
         suffix: str = ".png",
@@ -530,9 +655,17 @@ class ScoreTiler(GridTiler):
         self.scorer = scorer
         self.n_tiles = n_tiles
 
-        super().__init__(tile_size, level, check_tissue, pixel_overlap, prefix, suffix)
+        super().__init__(
+            tile_size,
+            level,
+            check_tissue,
+            tissue_percent,
+            pixel_overlap,
+            prefix,
+            suffix,
+        )
 
-    def extract(self, slide: Slide, report_path: str = None):
+    def extract(self, slide: Slide, report_path: str = None) -> None:
         """Extract grid tiles and save them to disk, according to a scoring function and
         following this filename pattern:
         `{prefix}tile_{tiles_counter}_level{level}_{x_ul_wsi}-{y_ul_wsi}-{x_br_wsi}-{y_br_wsi}{suffix}`
@@ -545,10 +678,18 @@ class ScoreTiler(GridTiler):
             Slide from which to extract the tiles
         report_path : str, optional
             Path to the CSV report. If None, no report will be saved
+
+        Raises
+        ------
+        TileSizeError
+            If the tile size is larger than the slide size
+        LevelError
+            If the level is not available for the slide
         """
-        highest_score_tiles, highest_scaled_score_tiles = self._highest_score_tiles(
-            slide
-        )
+        self._validate_level(slide)
+        self._validate_tile_size(slide)
+
+        highest_score_tiles, highest_scaled_score_tiles = self._tiles_generator(slide)
 
         tiles_counter = 0
         filenames = []
@@ -569,8 +710,10 @@ class ScoreTiler(GridTiler):
 
     # ------- implementation helpers -------
 
-    def _highest_score_tiles(self, slide: Slide) -> List[Tuple[float, CoordinatePair]]:
-        """Calculate the tiles with the highest scores and their extraction coordinates.
+    def _tiles_generator(
+        self, slide: Slide
+    ) -> Tuple[List[Tuple[float, CoordinatePair]], List[Tuple[float, CoordinatePair]]]:
+        r"""Calculate the tiles with the highest scores and their extraction coordinates
 
         Parameters
         ----------
@@ -579,19 +722,24 @@ class ScoreTiler(GridTiler):
 
         Returns
         -------
-        List[Tuple[float, CoordinatePair]]
-            List of tuples containing the score and the extraction coordinates for the
-            tiles with the highest score. Each tuple represents a tile.
-        List[Tuple[float, CoordinatePair]]
-            List of tuples containing the scaled score between 0 and 1 and the
-            extraction coordinates for the tiles with the highest score. Each tuple
-            represents a tile.
+        Tuple[List[Tuple[float, CoordinatePair]], List[Tuple[float, CoordinatePair]]]
+            List of tuples containing the scores and the extraction coordinates
+            for the tiles with the highest scores. If scaled=True, each score `s_i` of
+            the i-th tile is normalized as
+
+            .. math::
+
+                s_{\hat{i}}=\frac{s_i-\min_{j\in T}{s_j}}{\max_{j\in T}{s_j}-\min_{j\in T}{s_j}}
+
+            where `T` is the set of all the retrieved tiles. Notice that the normalized
+            scores range between 0 and 1. This could be useful to have a more intuitive
+            comparison between the scores. Each tuple represents a tile.
 
         Raises
         ------
         ValueError
             If ``n_tiles`` is negative.
-        """
+        """  # noqa
         all_scores = self._scores(slide)
         scaled_scores = self._scale_scores(all_scores)
 
@@ -611,8 +759,8 @@ class ScoreTiler(GridTiler):
 
         return highest_score_tiles, highest_scaled_score_tiles
 
+    @staticmethod
     def _save_report(
-        self,
         report_path: str,
         highest_score_tiles: List[Tuple[float, CoordinatePair]],
         highest_scaled_score_tiles: List[Tuple[float, CoordinatePair]],
@@ -624,33 +772,36 @@ class ScoreTiler(GridTiler):
 
         Parameters
         ----------
+        slide : Slide
+            The slide to extract the tiles from.
         report_path : str
             Path to the report
         highest_score_tiles : List[Tuple[float, CoordinatePair]]
             List of tuples containing the score and the extraction coordinates for the
             tiles with the highest score. Each tuple represents a tile.
-        List[Tuple[float, CoordinatePair]]
+        highest_scaled_score_tiles : List[Tuple[float, CoordinatePair]]
             List of tuples containing the scaled score between 0 and 1 and the
             extraction coordinates for the tiles with the highest score. Each tuple
             represents a tile.
         filenames : List[str]
             List of the tiles' filename
         """
-
         header = ["filename", "score", "scaled_score"]
         rows = [
             dict(zip(header, values))
             for values in zip(
                 filenames,
-                np.array(highest_score_tiles)[:, 0],
-                np.array(highest_scaled_score_tiles)[:, 0],
+                np.array(highest_score_tiles, dtype=object)[:, 0],
+                np.array(highest_scaled_score_tiles, dtype=object)[:, 0],
             )
         ]
 
-        with open(report_path, "w+", newline="") as f:
-            w = csv.DictWriter(f, fieldnames=header, lineterminator=os.linesep)
-            w.writeheader()
-            w.writerows(rows)
+        with open(report_path, "w+", newline="") as filename:
+            writer = csv.DictWriter(
+                filename, fieldnames=header, lineterminator=os.linesep
+            )
+            writer.writeheader()
+            writer.writerows(rows)
 
     @staticmethod
     def _scale_scores(
@@ -668,8 +819,8 @@ class ScoreTiler(GridTiler):
         List[Tuple[float, CoordinatePair]])
             Scaled scores
         """
-        scores_ = np.array(scores)[:, 0]
-        coords = np.array(scores)[:, 1]
+        scores_ = np.array(scores, dtype=object)[:, 0]
+        coords = np.array(scores, dtype=object)[:, 1]
         scores_scaled = (scores_ - np.min(scores_)) / (
             np.max(scores_) - np.min(scores_)
         )
@@ -690,12 +841,12 @@ class ScoreTiler(GridTiler):
             List of tuples containing the score and the extraction coordinates for each
             tile. Each tuple represents a tile.
         """
-        if next(self._grid_tiles_generator(slide), None) is None:
+        if next(super()._tiles_generator(slide), None) is None:
             raise RuntimeError(
                 "No tiles have been generated. This could happen if `check_tissue=True`"
             )
 
-        grid_tiles = self._grid_tiles_generator(slide)
+        grid_tiles = super()._tiles_generator(slide)
         scores = []
 
         for tile, tile_wsi_coords in grid_tiles:
