@@ -15,11 +15,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ------------------------------------------------------------------------
+
 import functools
 import warnings
-from collections import deque
-from itertools import filterfalse as ifilterfalse
-from typing import Callable, List, Tuple
+from typing import Any, Callable, List, Tuple
 
 import numpy as np
 import PIL
@@ -76,7 +75,11 @@ def np_to_pil(np_img: np.ndarray) -> PIL.Image.Image:
         return image_array.astype(np.uint8) * 255
 
     def _transform_float(image_array):
-        return (image_array * 255).astype(np.uint8)
+        return (
+            image_array.astype(np.uint8)
+            if np.max(image_array) > 1
+            else (image_array * 255).astype(np.uint8)
+        )
 
     types_factory = {
         "bool": _transform_bool(np_img),
@@ -208,14 +211,7 @@ def threshold_to_mask(
     return relate(img_arr, threshold)
 
 
-class Counter(dict):
-    """Mapping where default values are zero"""
-
-    def __missing__(self, key):
-        return 0
-
-
-class lazyproperty(object):
+def lazyproperty(f: Callable[..., Any]):
     """Decorator like @property, but evaluated only on first access.
 
     Like @property, this can only be used to decorate methods having only
@@ -242,18 +238,15 @@ class lazyproperty(object):
     code from any sequencing considerations; if it's accessed from more than
     one location, it's assured it will be ready whenever needed.
 
-    Loosely based on: https://stackoverflow.com/a/6849299/1902513.
-
     A lazyproperty is read-only. There is no counterpart to the optional
     "setter" (or deleter) behavior of an @property. This is critically
     important to maintaining its immutability and idempotence guarantees.
     Attempting to assign to a lazyproperty raises AttributeError
     unconditionally.
-
     The parameter names in the methods below correspond to this usage
     example::
 
-        class Obj(object)
+        class Obj(object):
 
             @lazyproperty
             def fget(self):
@@ -264,122 +257,5 @@ class lazyproperty(object):
     Not suitable for wrapping a function (as opposed to a method) because it
     is not callable.
     """
-
-    def __init__(self, fget):
-        """*fget* is the decorated method (a "getter" function).
-
-        A lazyproperty is read-only, so there is only an *fget* function (a
-        regular @property can also have an fset and fdel function). This name
-        was chosen for consistency with Python's `property` class which uses
-        this name for the corresponding parameter.
-        """
-        # ---maintain a reference to the wrapped getter method
-        self._fget = fget
-        # ---adopt fget's __name__, __doc__, and other attributes
-        functools.update_wrapper(self, fget)
-
-    def __get__(self, obj, type=None):
-        """Called on each access of 'fget' attribute on class or instance.
-
-        *self* is this instance of a lazyproperty descriptor "wrapping" the
-        property method it decorates (`fget`, nominally).
-
-        *obj* is the "host" object instance when the attribute is accessed
-        from an object instance, e.g. `obj = Obj(); obj.fget`. *obj* is None
-        when accessed on the class, e.g. `Obj.fget`.
-
-        *type* is the class hosting the decorated getter method (`fget`) on
-        both class and instance attribute access.
-        """
-        if obj is None:
-            return self
-
-        value = obj.__dict__.get(self.__name__)
-        if value is None:
-            value = self._fget(obj)
-            obj.__dict__[self.__name__] = value
-        return value
-
-    def __set__(self, obj, value):
-        raise AttributeError("can't set attribute")
-
-
-def lru_cache(maxsize=100):  # pragma: no cover
-    """Least-recently-used cache decorator.
-
-    Arguments to the cached function must be hashable.
-    Cache performance statistics stored in f.hits and f.misses.
-    Clear the cache with f.clear().
-    http://en.wikipedia.org/wiki/Cache_algorithms#Least_Recently_Used
-    """
-    maxqueue = maxsize * 10
-
-    def decorating_function(
-        user_function, len=len, iter=iter, tuple=tuple, sorted=sorted, KeyError=KeyError
-    ):
-        cache = {}  # mapping of args to results
-        queue = deque()  # order that keys have been used
-        refcount = Counter()  # times each key is in the queue
-        sentinel = object()  # marker for looping around the queue
-        kwd_mark = object()  # separate positional and keyword args
-
-        # lookup optimizations (ugly but fast)
-        queue_append, queue_popleft = queue.append, queue.popleft
-        queue_appendleft, queue_pop = queue.appendleft, queue.pop
-
-        @functools.wraps(user_function)
-        def wrapper(*args, **kwds):
-            # cache key records both positional and keyword args
-            key = args
-            if kwds:
-                key += (kwd_mark,) + tuple(sorted(kwds.items()))
-
-            # record recent use of this key
-            queue_append(key)
-            refcount[key] += 1
-
-            # get cache entry or compute if not found
-            try:
-                result = cache[key]
-                wrapper.hits += 1
-            except KeyError:
-                result = user_function(*args, **kwds)
-                cache[key] = result
-                wrapper.misses += 1
-
-                # purge least recently used cache entry
-                if len(cache) > maxsize:
-                    key = queue_popleft()
-                    refcount[key] -= 1
-                    while refcount[key]:
-                        key = queue_popleft()
-                        refcount[key] -= 1
-                    del cache[key], refcount[key]
-
-            # periodically compact the queue by eliminating duplicate keys
-            # while preserving order of most recent access
-            if len(queue) > maxqueue:
-                refcount.clear()
-                queue_appendleft(sentinel)
-                for key in ifilterfalse(
-                    refcount.__contains__, iter(queue_pop, sentinel)
-                ):
-                    queue_appendleft(key)
-                    refcount[key] = 1
-
-            return result
-
-        def clear():  # pragma: no cover
-            cache.clear()
-            queue.clear()
-            refcount.clear()
-            wrapper.hits = wrapper.misses = 0
-
-        wrapper.hits = wrapper.misses = 0
-        wrapper.clear = clear
-        return wrapper
-
-    return decorating_function
-
-
-memoize = lru_cache(100)
+    # pylint: disable=unused-variable
+    return property(functools.lru_cache(maxsize=100)(f))
