@@ -8,7 +8,7 @@ import numpy as np
 from PIL import Image
 
 
-class LazyResponder(object):
+class LazyResponder:
     """Loads and caches fixtures files by name from fixture directory.
     Provides access to all the svs fixtures in a directory by
     a standardized mapping of the file name, e.g. ca-1-c.svs is available
@@ -87,5 +87,6 @@ SVS = LazySVSResponseLoader("./svs-images")
 RGBA = LazyPILResponseLoader("./pil-images-rgba")
 RGB = LazyPILResponseLoader("./pil-images-rgb")
 GS = LazyPILResponseLoader("./pil-images-gs")
-NPY = LazyNPYResponseLoader("./mask-arrays")
+MASKNPY = LazyNPYResponseLoader("./mask-arrays")
+NPY = LazyNPYResponseLoader("./arrays")
 TILES = LazyPILResponseLoader("./tiles")
