@@ -3,7 +3,7 @@ import pytest
 
 import histolab.filters.morphological_filters_functional as mof
 
-from ..fixtures import NPY
+from ..fixtures import MASKNPY
 from ..util import load_expectation
 
 
@@ -11,7 +11,7 @@ from ..util import load_expectation
     "mask_array, min_size, avoid_overmask, overmask_thresh, expected_array",
     (
         (
-            NPY.DIAGNOSTIC_SLIDE_THUMB_RGB1_HYSTERESIS_THRESHOLD_MASK,
+            MASKNPY.DIAGNOSTIC_SLIDE_THUMB_RGB1_HYSTERESIS_THRESHOLD_MASK,
             3000,
             True,
             95,
@@ -19,7 +19,7 @@ from ..util import load_expectation
             "objects-mask",
         ),
         (
-            NPY.DIAGNOSTIC_SLIDE_THUMB_HSV_OTSU_THRESHOLD_MASK,
+            MASKNPY.DIAGNOSTIC_SLIDE_THUMB_HSV_OTSU_THRESHOLD_MASK,
             3000,
             True,
             95,
@@ -27,7 +27,7 @@ from ..util import load_expectation
             "-mask",
         ),
         (
-            NPY.DIAGNOSTIC_SLIDE_THUMB_HSV_OTSU_THRESHOLD_MASK,
+            MASKNPY.DIAGNOSTIC_SLIDE_THUMB_HSV_OTSU_THRESHOLD_MASK,
             3000,
             False,
             95,
@@ -35,7 +35,7 @@ from ..util import load_expectation
             "2-mask",
         ),
         (
-            NPY.DIAGNOSTIC_SLIDE_THUMB_RGB1_HYSTERESIS_THRESHOLD_MASK,
+            MASKNPY.DIAGNOSTIC_SLIDE_THUMB_RGB1_HYSTERESIS_THRESHOLD_MASK,
             1200,
             True,
             25,
@@ -54,3 +54,21 @@ def test_remove_small_objects_filter(
 
     np.testing.assert_array_equal(mask_no_small_object, expected_value)
     assert type(mask_no_small_object) == np.ndarray
+
+
+@pytest.mark.parametrize(
+    "mask_array, region_shape, expected_array",
+    (
+        (MASKNPY.YTMA1, 6, "mask-arrays/ytma1-watershed-segmentation-region6"),
+        (MASKNPY.YTMA2, 6, "mask-arrays/ytma2-watershed-segmentation-region6"),
+        (MASKNPY.YTMA1, 3, "mask-arrays/ytma1-watershed-segmentation-region3"),
+        (MASKNPY.YTMA2, 3, "mask-arrays/ytma2-watershed-segmentation-region3"),
+    ),
+)
+def test_watershed_segmentation_filter(mask_array, region_shape, expected_array):
+    expected_value = load_expectation(expected_array, type_="npy")
+
+    mask_watershed = mof.watershed_segmentation(mask_array, region_shape)
+
+    np.testing.assert_array_equal(mask_watershed, expected_value)
+    assert type(mask_watershed) == np.ndarray
