@@ -1,12 +1,12 @@
 import pytest
 
 from histolab.tile import Tile
-from histolab.types import CoordinatePair
+from histolab.types import CP
 
 from ..fixtures import TILES
 
 
-class Describe_Tile(object):
+class Describe_Tile:
     @pytest.mark.parametrize(
         "tile_img, expected_result",
         (
@@ -21,9 +21,23 @@ class Describe_Tile(object):
         ),
     )
     def it_knows_if_is_is_almost_white(self, tile_img, expected_result):
-        coords = CoordinatePair(0, 512, 0, 512)
+        coords = CP(0, 512, 0, 512)
         tile = Tile(tile_img, coords)
 
         is_almost_white = tile._is_almost_white
 
         assert is_almost_white == expected_result
+
+    @pytest.mark.parametrize(
+        "tile_image, expected_value",
+        (
+            (TILES.LIVER_LEVEl2_10907_7808_11707_8608, False),  # all tissue
+            (TILES.LIVER_LEVEl2_20914_13715_21714_14515, True),  # some tissue
+            (TILES.LIVER_LEVEl2_57138_8209_57938_9009, True),  # some tissue
+            (TILES.LIVER_LEVEl2_38626_13514_39426_14315, False),  # no tissue
+        ),
+    )
+    def it_knows_if_it_has_only_some_tissue(self, tile_image, expected_value):
+        tile = Tile(tile_image, CP(5, 5, 5, 5))
+
+        assert tile._has_only_some_tissue() == expected_value
