@@ -4,7 +4,6 @@ import errno
 import math
 import os
 from collections import namedtuple
-from unittest.mock import call
 
 import numpy as np
 import openslide
@@ -16,12 +15,13 @@ from histolab.exceptions import LevelError
 from histolab.filters.compositions import _SlideFiltersComposition
 from histolab.filters.image_filters import Compose
 from histolab.slide import Slide, SlideSet
-from histolab.types import CoordinatePair, Region
+from histolab.types import CP, Region
 from histolab.util import regions_from_binary_mask
 
 from ..unitutil import (
     ANY,
-    PILImageMock,
+    PILIMG,
+    base_test_slide,
     class_mock,
     dict_list_eq,
     function_mock,
@@ -34,7 +34,7 @@ from ..unitutil import (
 )
 
 
-class Describe_Slide(object):
+class Describe_Slide:
     def it_constructs_from_args(self, request):
         _init_ = initializer_mock(request, Slide)
         _slide_path = "/foo/bar/myslide.svs"
@@ -71,33 +71,115 @@ class Describe_Slide(object):
             str(err.value) == "expected str, bytes or os.PathLike object, not NoneType"
         )
 
-    def it_generates_the_correct_breadcumb(self, request, breadcumb_fixture):
+    @pytest.mark.parametrize(
+        "resampled_dims, dir_path, slide_path, proc_path, scale_factor, expected_path",
         (
-            resampled_dims,
-            dir_path,
-            slide_path,
-            proc_path,
-            scale_factor,
-            expected_path,
-        ) = breadcumb_fixture
+            (
+                (245, 123, 145, 99),
+                "/foo/bar/b/0/9",
+                "/foo/bar/myslide.svs",
+                "processed",
+                64,
+                os.path.join("/foo/bar/b/0/9", "myslide-64x-245x123-145x99.png"),
+            ),
+            (
+                (245, 123, 145, 99),
+                "/foo/bar/b/0/9",
+                "/foo/bar/myslide.svs",
+                "processed",
+                32,
+                os.path.join("/foo/bar/b/0/9", "myslide-32x-245x123-145x99.png"),
+            ),
+            (
+                (None, None, None, None),
+                "/foo/bar/b/0/9",
+                "/foo/bar/myslide.svs",
+                "processed",
+                64,
+                os.path.join("/foo/bar/b/0/9", "myslide*.png"),
+            ),
+            (
+                (None, 234, 192, None),
+                "/foo/bar/b/0/9",
+                "/foo/bar/myslide.svs",
+                "processed",
+                64,
+                os.path.join("/foo/bar/b/0/9", "myslide-64x-Nonex234-192xNone.png"),
+            ),
+            (
+                (123, 234, 192, None),
+                "/foo/bar/b/0/9",
+                "/foo/bar/myslide.svs",
+                "processed",
+                64,
+                os.path.join("/foo/bar/b/0/9", "myslide-64x-123x234-192xNone.png"),
+            ),
+            (
+                (None, None, 192, None),
+                "/foo/bar/b/0/9",
+                "/foo/bar/myslide.svs",
+                "processed",
+                64,
+                os.path.join("/foo/bar/b/0/9", "myslide-64x-NonexNone-192xNone.png"),
+            ),
+        ),
+    )
+    def it_generates_the_correct_breadcrumb(
+        self,
+        request,
+        resampled_dims,
+        dir_path,
+        slide_path,
+        proc_path,
+        scale_factor,
+        expected_path,
+    ):
         _resampled_dimensions = method_mock(request, Slide, "_resampled_dimensions")
         _resampled_dimensions.return_value = resampled_dims
         slide = Slide(slide_path, proc_path)
 
-        _breadcumb = slide._breadcumb(dir_path, scale_factor)
+        _breadcrumb = slide._breadcrumb(dir_path, scale_factor)
 
-        assert _breadcumb == expected_path
+        assert _breadcrumb == expected_path
 
-    def it_knows_its_name(self, slide_name_fixture):
-        _slide_path, expected_value = slide_name_fixture
-        slide = Slide(_slide_path, "processed/")
+    @pytest.mark.parametrize(
+        "slide_path, expected_value",
+        (("/foo/bar/myslide.svs", "myslide"), ("/foo/myslide.svs", "myslide")),
+    )
+    def it_knows_its_name(self, slide_path, expected_value):
+        slide = Slide(slide_path, "processed/")
 
         name = slide.name
 
         assert name == expected_value
 
-    def it_knows_its_scaled_image_path(self, scaled_img_path_fixture, resampled_dims_):
-        slide_path, proc_path, slide_dims, expected_value = scaled_img_path_fixture
+    @pytest.mark.parametrize(
+        "slide_path, proc_path, slide_dims, expected_value",
+        (
+            (
+                "1.svs",
+                "1/p",
+                (345, 111, 333, 444),
+                os.path.join("1/p", "1-22x-345x111-333x444.png"),
+            ),
+            (
+                "2.svs",
+                "2/p",
+                (345, 111, None, None),
+                os.path.join("2/p", "2-22x-345x111-NonexNone.png"),
+            ),
+            (
+                "2.svs",
+                "2/p",
+                (345, 111, 123, 123),
+                os.path.join("2/p", "2-22x-345x111-123x123.png"),
+            ),
+            ("2.svs", "2/p", (None, None, None, None), os.path.join("2/p", "2*.png")),
+        ),
+    )
+    def it_knows_its_scaled_image_path(
+        self, resampled_dims_, slide_path, proc_path, slide_dims, expected_value
+    ):
         resampled_dims_.return_value = slide_dims
         slide = Slide(slide_path, proc_path)
 
@@ -120,11 +202,7 @@ class Describe_Slide(object):
         assert thumbnail_path == expected_value
 
     def it_knows_its_dimensions(self, tmpdir):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
 
         slide_dims = slide.dimensions
 
@@ -159,11 +237,7 @@ class Describe_Slide(object):
         assert str(err.value) == "division by zero"
 
     def it_knows_its_resampled_array(self, tmpdir, resampled_dims_):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
         resampled_dims_.return_value = (100, 200, 300, 400)
 
         resampled_array = slide.resampled_array(scale_factor=32)
@@ -172,22 +246,14 @@ class Describe_Slide(object):
         assert resampled_array.shape == (400, 300, 3)
 
     def it_knows_its_thumbnail_size(self, tmpdir):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
 
         thumb_size = slide._thumbnail_size
 
         assert thumb_size == (500, 500)
 
     def it_creates_a_correct_slide_object(self, tmpdir):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_50X50_RGBA_COLOR_155_0_0
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_50X50_155_0_0)
 
         _wsi = slide._wsi
 
@@ -210,16 +276,11 @@ class Describe_Slide(object):
 
         assert isinstance(err.value, PIL.UnidentifiedImageError)
         assert (
-            str(err.value) == "cannot identify image file "
-            f"{repr(os.path.join(slide_path))}"
+            str(err.value) == "Your wsi has something broken inside, a doctor is needed"
         )
 
     def it_can_resample_itself(self, tmpdir, resampled_dims_):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
         resampled_dims_.return_value = (100, 200, 300, 400)
 
         _resample = slide._resample(32)
@@ -244,11 +305,7 @@ class Describe_Slide(object):
         assert _resample[0].mode == "RGB"
 
     def it_resamples_with_the_correct_scale_factor(self, tmpdir, resampled_dims_):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
         resampled_dims_.return_value = (500, 500, 15, 15)
 
         _resample = slide._resample(32)
@@ -257,7 +314,7 @@ class Describe_Slide(object):
 
     def it_can_save_scaled_image(self, tmpdir, resampled_dims_):
         tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
         image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
         slide_path = os.path.join(tmp_path_, "mywsi.png")
         slide = Slide(slide_path, os.path.join(tmp_path_, "processed"))
@@ -272,7 +329,7 @@ class Describe_Slide(object):
 
     def it_can_save_thumbnail(self, tmpdir, resampled_dims_):
         tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
         image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
         slide_path = os.path.join(tmp_path_, "mywsi.png")
         slide = Slide(slide_path, os.path.join(tmp_path_, "processed"))
@@ -343,11 +400,7 @@ class Describe_Slide(object):
         RemoveSmallHoles_,
         RemoveSmallObjects_,
     ):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
         regions = [Region(index=0, area=33, bbox=(0, 0, 2, 2), center=(0.5, 0.5))]
         main_tissue_areas_mask_filters_ = property_mock(
             request, _SlideFiltersComposition, "tissue_mask_filters"
@@ -365,27 +418,28 @@ class Describe_Slide(object):
             request, "histolab.slide.regions_from_binary_mask"
         )
         regions_from_binary_mask.return_value = regions
-        biggest_regions_ = function_mock(
-            request, "histolab.slide.Slide._biggest_regions"
+        biggest_regions_ = method_mock(
+            request, Slide, "_biggest_regions", autospec=False
         )
         biggest_regions_.return_value = regions
         region_coordinates_ = function_mock(
             request, "histolab.slide.region_coordinates"
         )
-        region_coordinates_.return_values = CoordinatePair(0, 0, 2, 2)
+        region_coordinates_.return_values = CP(0, 0, 2, 2)
         polygon_to_mask_array_ = function_mock(
             request, "histolab.util.polygon_to_mask_array"
         )
-        polygon_to_mask_array_(
-            (1000, 1000), CoordinatePair(0, 0, 2, 2)
-        ).return_value = [[True, True], [False, True]]
+        polygon_to_mask_array_((1000, 1000), CP(0, 0, 2, 2)).return_value = [
+            [True, True],
+            [False, True],
+        ]
 
         biggest_mask_tissue_box = slide.biggest_tissue_box_mask
 
         region_coordinates_.assert_called_once_with(regions[0])
-        biggest_regions_.assert_called_once_with(slide, regions, n=1)
+        biggest_regions_.assert_called_once_with(regions, n=1)
         polygon_to_mask_array_.assert_called_once_with(
-            (1000, 1000), CoordinatePair(x_ul=0, y_ul=0, x_br=2, y_br=2)
+            (1000, 1000), CP(x_ul=0, y_ul=0, x_br=2, y_br=2)
         )
         np.testing.assert_almost_equal(biggest_mask_tissue_box, np.zeros((500, 500)))
 
@@ -393,11 +447,7 @@ class Describe_Slide(object):
         not on_ci() or is_win32(), reason="Only run on CIs; hangs on Windows CIs"
     )
     def it_can_show_its_thumbnail(self, tmpdir):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
 
         slide.save_thumbnail()
 
@@ -415,36 +465,39 @@ class Describe_Slide(object):
             f"directory: {repr(os.path.join('processed', 'thumbnails', 'b.png'))}"
         )
 
-    def it_knows_its_level_dimensions(self, tmpdir):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+    @pytest.mark.parametrize(
+        "level, expected_value", ((0, (500, 500)), (-1, (500, 500)))
+    )
+    def it_knows_its_level_dimensions(self, level, expected_value, tmpdir):
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
 
-        level_dimensions = slide.level_dimensions(level=0)
+        level_dimensions = slide.level_dimensions(level=level)
 
-        assert level_dimensions == (500, 500)
+        assert level_dimensions == expected_value
+
+    @pytest.mark.parametrize("level", (3, -3))
+    def but_it_raises_expection_when_level_does_not_exist(self, level, tmpdir):
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
 
-    def but_it_raises_expection_when_level_does_not_exist(self, tmpdir):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
         with pytest.raises(LevelError) as err:
-            slide.level_dimensions(level=3)
+            slide.level_dimensions(level=level)
 
         assert isinstance(err.value, LevelError)
-        assert str(err.value) == "Level 3 not available. Number of available levels: 1"
+        assert (
+            str(err.value)
+            == f"Level {level} not available. Number of available levels: 1"
+        )
 
-    def it_knows_if_coords_are_valid(self, valid_coords_fixture, tmpdir):
-        coords, expected_result = valid_coords_fixture
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+    @pytest.mark.parametrize(
+        "coords, expected_result",
+        (
+            (CP(0, 128, 0, 128), True),
+            (CP(800000, 90000, 8000010, 90010), False),
+            (CP(800000, 90000, -1, 90010), False),
+        ),
+    )
+    def it_knows_if_coords_are_valid(self, coords, expected_result, tmpdir):
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
 
         _are_valid = slide._has_valid_coords(coords)
 
@@ -452,152 +505,46 @@ class Describe_Slide(object):
         assert _are_valid == expected_result
 
     def it_knows_its_levels(self, tmpdir):
-        tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
-        image.save(os.path.join(tmp_path_, "mywsi.png"), "PNG")
-        slide_path = os.path.join(tmp_path_, "mywsi.png")
-        slide = Slide(slide_path, "processed")
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGBA_COLOR_500X500_155_249_240)
 
         levels = slide.levels
 
         assert type(levels) == list
         assert levels == [0]
 
-    # fixtures -------------------------------------------------------
+    def it_can_access_to_its_properties(self, request):
+        slide = Slide("path", "processed")
+        properties = property_mock(request, Slide, "properties")
+        properties.return_value = {"foo": "bar"}
 
-    @pytest.fixture(params=[("/a/b/mywsi.svs", ".svs"), ("/a/b/mywsi.34s", ".34s")])
-    def slide_ext_fixture(self, request):
-        slide_path, expected_value = request.param
-        return slide_path, expected_value
+        assert slide.properties == {"foo": "bar"}
 
-    @pytest.fixture(
-        params=[
-            (
-                "/foo/bar/myslide.svs",
-                "/foo/bar/myslide/processed",
-                (345, 111, 333, 444),
-                os.path.join(
-                    "/foo/bar/myslide/processed", "myslide-22x-345x111-333x444.png"
-                ),
-            ),
-            (
-                "/foo/bar/myslide2.svs",
-                "/foo/bar/myslide/processed",
-                (345, 111, None, None),
-                os.path.join(
-                    "/foo/bar/myslide/processed", "myslide2-22x-345x111-NonexNone.png"
-                ),
-            ),
-            (
-                "/foo/bar/myslide2.svs",
-                "/foo/bar/myslide/processed",
-                (345, 111, 123, 123),
-                os.path.join(
-                    "/foo/bar/myslide/processed", "myslide2-22x-345x111-123x123.png"
-                ),
-            ),
-            (
-                "/foo/bar/myslide2.svs",
-                "/foo/bar/myslide/processed",
-                (None, None, None, None),
-                os.path.join("/foo/bar/myslide/processed", "myslide2*.png"),
-            ),
-        ]
-    )
-    def scaled_img_path_fixture(self, request):
-        slide_path, proc_path, slide_dims, expected_value = request.param
-        return slide_path, proc_path, slide_dims, expected_value
+    @pytest.mark.parametrize("level, expected_value", ((-1, 8), (-2, 7), (-9, 0)))
+    def it_can_remap_negative_level_indices(self, level, expected_value, levels_prop):
+        levels_prop.return_value = [0, 1, 2, 3, 4, 5, 6, 7, 8]
 
-    @pytest.fixture(
-        params=[
-            (
-                (245, 123, 145, 99),
-                "/foo/bar/b/0/9",
-                "/foo/bar/myslide.svs",
-                "processed",
-                64,
-                os.path.join("/foo/bar/b/0/9", "myslide-64x-245x123-145x99.png"),
-            ),
-            (
-                (245, 123, 145, 99),
-                "/foo/bar/b/0/9",
-                "/foo/bar/myslide.svs",
-                "processed",
-                32,
-                os.path.join("/foo/bar/b/0/9", "myslide-32x-245x123-145x99.png"),
-            ),
-            (
-                (None, None, None, None),
-                "/foo/bar/b/0/9",
-                "/foo/bar/myslide.svs",
-                "processed",
-                64,
-                os.path.join("/foo/bar/b/0/9", "myslide*.png"),
-            ),
-            (
-                (None, 234, 192, None),
-                "/foo/bar/b/0/9",
-                "/foo/bar/myslide.svs",
-                "processed",
-                64,
-                os.path.join("/foo/bar/b/0/9", "myslide-64x-Nonex234-192xNone.png"),
-            ),
-            (
-                (123, 234, 192, None),
-                "/foo/bar/b/0/9",
-                "/foo/bar/myslide.svs",
-                "processed",
-                64,
-                os.path.join("/foo/bar/b/0/9", "myslide-64x-123x234-192xNone.png"),
-            ),
-            (
-                (None, None, 192, None),
-                "/foo/bar/b/0/9",
-                "/foo/bar/myslide.svs",
-                "processed",
-                64,
-                os.path.join("/foo/bar/b/0/9", "myslide-64x-NonexNone-192xNone.png"),
-            ),
-        ]
-    )
-    def breadcumb_fixture(self, request):
-        (
-            resampled_dims,
-            dir_path,
-            slide_path,
-            proc_path,
-            scale_factor,
-            expected_path,
-        ) = request.param
-        return (
-            resampled_dims,
-            dir_path,
-            slide_path,
-            proc_path,
-            scale_factor,
-            expected_path,
+        slide = Slide("path", "processed")
+
+        assert slide._remap_level(level) == expected_value
+
+    def but_it_raises_a_level_error_when_it_cannot_be_mapped(self, tmpdir, levels_prop):
+        levels_prop.return_value = [0, 1, 2, 3, 4, 5, 6, 7, 8]
+        slide, _ = base_test_slide(tmpdir, PILIMG.RGB_RANDOM_COLOR_500X500)
+
+        with pytest.raises(LevelError) as err:
+            slide._remap_level(-10)
+
+        assert isinstance(err.value, LevelError)
+        assert (
+            str(err.value) == "Level -10 not available. Number of available levels: 1"
         )
 
-    @pytest.fixture(
-        params=[("/foo/bar/myslide.svs", "myslide"), ("/foo/myslide.svs", "myslide")]
-    )
-    def slide_name_fixture(self, request):
-        slide_path, expceted_value = request.param
-        return slide_path, expceted_value
-
-    @pytest.fixture(
-        params=[
-            (CoordinatePair(0, 128, 0, 128), True),
-            (CoordinatePair(800000, 90000, 8000010, 90010), False),
-            (CoordinatePair(800000, 90000, -1, 90010), False),
-        ]
-    )
-    def valid_coords_fixture(self, request):
-        coords, expected_result = request.param
-        return coords, expected_result
-
     # fixture components ---------------------------------------------
 
+    @pytest.fixture
+    def levels_prop(self, request):
+        return property_mock(request, Slide, "levels")
+
     @pytest.fixture
     def resampled_dims_(self, request):
         return method_mock(request, Slide, "_resampled_dimensions")
@@ -633,7 +580,7 @@ class Describe_Slide(object):
         )
 
 
-class Describe_Slideset(object):
+class Describe_Slideset:
     def it_constructs_from_args(self, request):
         _init_ = initializer_mock(request, SlideSet)
         _slides_path = "/foo/bar/"
@@ -647,70 +594,80 @@ class Describe_Slideset(object):
         )
         assert isinstance(slideset, SlideSet)
 
-    def it_can_constructs_slides(self, request, tmpdir):
+    def it_can_constructs_slides(self, request, tmpdir, Slide_):
         tmp_path_ = tmpdir.mkdir("myslide")
-        slides_ = tuple(instance_mock(request, Slide) for _ in range(10))
-        _slides_ = property_mock(request, SlideSet, "slides")
-        _slides_.return_value = slides_
+        slides_ = method_mock(request, SlideSet, "__iter__")
+        slides_.return_value = [Slide_ for _ in range(10)]
         slideset = SlideSet(tmp_path_, os.path.join(tmp_path_, "b"), [".svs"])
 
-        slides = slideset.slides
+        slides = slideset.__iter__()
 
-        _slides_.assert_called_once_with()
+        slides_.assert_called_once_with(slideset)
         assert len(slides) == 10
 
     def it_knows_its_slides(self, tmpdir):
         tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
         image.save(os.path.join(tmp_path_, "mywsi.svs"), "TIFF")
         slideset = SlideSet(tmp_path_, "proc", [".svs"])
 
-        slides = slideset.slides
-
-        assert len(slides) == 1
+        assert len(slideset) == 1
 
         slideset = SlideSet(None, "proc", [".svs"])
 
-        slides = slideset.slides
-
-        assert len(slides) == 0
+        assert len(slideset) == 0
 
         with pytest.raises(FileNotFoundError) as err:
             slideset = SlideSet("fake/path", "proc", [".svs"])
-            slides = slideset.slides
+            list(slideset)
 
         assert isinstance(err.value, FileNotFoundError)
         assert err.value.errno == errno.ENOENT
 
-    def it_constructs_its_sequence_of_slides_to_help(self, request, Slide_, tmpdir):
-        slides_path = tmpdir.mkdir("mypath")
-        for i in range(4):
-            open(os.path.join(slides_path, f"myfile{i}.svs"), "a")
-        slides_ = tuple(instance_mock(request, Slide) for _ in range(4))
-        Slide_.side_effect = iter(slides_)
-        slide_set = SlideSet(
-            slides_path=slides_path,
-            processed_path=os.path.join(slides_path, "processed"),
-            valid_extensions=[".svs"],
-        )
-        slides = tuple(slide_set.slides)
+    def it_can_access_directly_to_the_slides(self, request, Slide_):
+        slideset = instance_mock(request, SlideSet)
+        slideset.__iter__.side_effect = iter([Slide_])
 
-        assert sorted(Slide_.call_args_list) == sorted(
-            [
-                call(
-                    os.path.join(slides_path, f"myfile{i}.svs"),
-                    os.path.join(slides_path, "processed"),
-                )
-                for i in range(4)
-            ]
+        slideset[0]
+
+        slideset.__getitem__.assert_called_once_with(0)
+
+    def and_it_is_exaclty_what_expected(self, tmpdir):
+        tmp_path_ = tmpdir.mkdir("myslide")
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
+        image.save(os.path.join(tmp_path_, "mywsi.svs"), "TIFF")
+        slideset = SlideSet(tmp_path_, "proc", [".svs"])
+
+        slide = slideset[0]
+
+        np.testing.assert_array_almost_equal(
+            slide.resampled_array(), slideset[0].resampled_array()
         )
-        assert slides == slides_
+
+    def it_constructs_its_sequence_of_slides_to_help(self, tmpdir):
+        tmp_path_ = tmpdir.mkdir("myslide")
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
+        image.save(os.path.join(tmp_path_, "mywsi.svs"), "TIFF")
+        image2 = PILIMG.RGBA_COLOR_50X50_155_0_0
+        image2.save(os.path.join(tmp_path_, "mywsi2.svs"), "TIFF")
+        slideset = SlideSet(tmp_path_, "proc", [".svs"])
+        expected_slides = [
+            Slide(os.path.join(tmp_path_, _path), "proc")
+            for _path in os.listdir(tmp_path_)
+        ]
+
+        slides = slideset.__iter__()
+
+        for i, slide in enumerate(slides):
+            np.testing.assert_array_almost_equal(
+                slide.resampled_array(), expected_slides[i].resampled_array()
+            )
 
     def it_knows_the_slides_dimensions(self, tmpdir):
         tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
         image.save(os.path.join(tmp_path_, "mywsi.svs"), "TIFF")
-        image2 = PILImageMock.DIMS_50X50_RGBA_COLOR_155_0_0
+        image2 = PILIMG.RGBA_COLOR_50X50_155_0_0
         image2.save(os.path.join(tmp_path_, "mywsi2.svs"), "TIFF")
         slideset = SlideSet(tmp_path_, "proc", [".svs"])
 
@@ -724,9 +681,9 @@ class Describe_Slideset(object):
 
     def it_knows_its_slides_dimensions_list(self, tmpdir):
         tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
         image.save(os.path.join(tmp_path_, "mywsi.svs"), "TIFF")
-        image2 = PILImageMock.DIMS_50X50_RGBA_COLOR_155_0_0
+        image2 = PILIMG.RGBA_COLOR_50X50_155_0_0
         image2.save(os.path.join(tmp_path_, "mywsi2.svs"), "TIFF")
         slideset = SlideSet(tmp_path_, "proc", [".svs"])
 
@@ -735,11 +692,11 @@ class Describe_Slideset(object):
         assert sorted(_slides_dimensions_list) == sorted([(500, 500), (50, 50)])
 
     def it_knows_its_total_slides(self, request, Slide_):
-        slides = property_mock(request, SlideSet, "slides")
+        slides = method_mock(request, SlideSet, "__iter__")
         slides.return_value = [Slide_ for _ in range(4)]
         slideset = SlideSet("the/path", "proc", [".svs"])
 
-        total_slides = slideset.total_slides
+        total_slides = len(slideset)
 
         assert total_slides == 4
 
@@ -863,7 +820,7 @@ class Describe_Slideset(object):
         slide2 = instance_mock(request, Slide)
 
         slideset = SlideSet(tmp_path_, os.path.join(tmp_path_, "processed"), [])
-        slides = property_mock(request, SlideSet, "slides")
+        slides = method_mock(request, SlideSet, "__iter__")
         slides.return_value = [slide1, slide2]
         slideset.save_scaled_slides(32, 2)
 
@@ -876,7 +833,7 @@ class Describe_Slideset(object):
         slide2 = instance_mock(request, Slide)
 
         slideset = SlideSet(tmp_path_, os.path.join(tmp_path_, "processed"), [])
-        slides = property_mock(request, SlideSet, "slides")
+        slides = method_mock(request, SlideSet, "__iter__")
         slides.return_value = [slide1, slide2]
         slideset.save_thumbnails(2)
 
@@ -886,9 +843,9 @@ class Describe_Slideset(object):
     def it_generates_slides_stats(self, total_slides_prop, tmpdir):
         total_slides_prop.return_value = 2
         tmp_path_ = tmpdir.mkdir("myslide")
-        image = PILImageMock.DIMS_500X500_RGBA_COLOR_155_249_240
+        image = PILIMG.RGBA_COLOR_500X500_155_249_240
         image.save(os.path.join(tmp_path_, "mywsi.svs"), "TIFF")
-        image2 = PILImageMock.DIMS_50X50_RGBA_COLOR_155_0_0
+        image2 = PILIMG.RGBA_COLOR_50X50_155_0_0
         image2.save(os.path.join(tmp_path_, "mywsi2.svs"), "TIFF")
         slideset = SlideSet(tmp_path_, os.path.join("proc"), [".svs"])
 
